@@ -80,8 +80,6 @@
 #define PyUnicode_FromString PyString_FromString
 #endif
 
-using namespace tinyxml2;
-
 // TODO:
 //   recrawl on:
 //     package not found in cache
@@ -116,7 +114,7 @@ static const int MAX_CRAWL_DEPTH = 1000;
 static const int MAX_DEPENDENCY_DEPTH = 1000;
 static const double DEFAULT_MAX_CACHE_AGE = 60.0;
 
-XMLElement* get_manifest_root(Stackage* stackage);
+tinyxml2::XMLElement* get_manifest_root(Stackage* stackage);
 double time_since_epoch();
 
 #ifdef __APPLE__
@@ -153,7 +151,7 @@ class Stackage
     // \brief have we already loaded the manifest?
     bool manifest_loaded_;
     // \brief TinyXML structure, filled in during parsing
-    XMLDocument manifest_;
+    tinyxml2::XMLDocument manifest_;
     std::vector<Stackage*> deps_;
     bool deps_computed_;
     bool is_wet_package_;
@@ -168,6 +166,7 @@ class Stackage
             manifest_path_(manifest_path),
             manifest_name_(manifest_name),
             manifest_loaded_(false),
+            manifest_(true, tinyxml2::COLLAPSE_WHITESPACE),
             deps_computed_(false),
             is_metapackage_(false)
     {
@@ -179,20 +178,20 @@ class Stackage
       assert(is_wet_package_);
       assert(manifest_loaded_);
       // get name from package.xml instead of folder name
-      XMLElement* root = get_manifest_root(this);
-      for(XMLElement* el = root->FirstChildElement("name"); el; el = el->NextSiblingElement("name"))
+      tinyxml2::XMLElement* root = get_manifest_root(this);
+      for(tinyxml2::XMLElement* el = root->FirstChildElement("name"); el; el = el->NextSiblingElement("name"))
       {
         name_ = el->GetText();
         break;
       }
       // Get license texts, where there may be multiple elements for.
       std::string tagname_license = "license";
-      for(XMLElement* el = root->FirstChildElement(tagname_license.c_str()); el; el = el->NextSiblingElement(tagname_license.c_str()))
+      for(tinyxml2::XMLElement* el = root->FirstChildElement(tagname_license.c_str()); el; el = el->NextSiblingElement(tagname_license.c_str()))
       {
         licenses_.push_back(el->GetText());
       }
       // check if package is a metapackage
-      for(XMLElement* el = root->FirstChildElement("export"); el; el = el->NextSiblingElement("export"))
+      for(tinyxml2::XMLElement* el = root->FirstChildElement("export"); el; el = el->NextSiblingElement("export"))
       {
         if(el->FirstChildElement("metapackage"))
         {
@@ -736,8 +735,8 @@ Rosstackage::rosdeps(const std::string& name, bool direct,
 void
 Rosstackage::_rosdeps(Stackage* stackage, std::set<std::string>& rosdeps, const char* tag_name)
 {
-  XMLElement* root = get_manifest_root(stackage);
-  for(XMLElement* ele = root->FirstChildElement(tag_name);
+  tinyxml2::XMLElement* root = get_manifest_root(stackage);
+  for(tinyxml2::XMLElement* ele = root->FirstChildElement(tag_name);
       ele;
       ele = ele->NextSiblingElement(tag_name))
   {
@@ -779,8 +778,8 @@ Rosstackage::vcs(const std::string& name, bool direct,
         it != deps_vec.end();
         ++it)
     {
-      XMLElement* root = get_manifest_root(*it);
-      for(XMLElement* ele = root->FirstChildElement(MANIFEST_TAG_VERSIONCONTROL);
+      tinyxml2::XMLElement* root = get_manifest_root(*it);
+      for(tinyxml2::XMLElement* ele = root->FirstChildElement(MANIFEST_TAG_VERSIONCONTROL);
           ele;
           ele = ele->NextSiblingElement(MANIFEST_TAG_VERSIONCONTROL))
       {
@@ -1021,14 +1020,14 @@ Rosstackage::exports_dry_package(Stackage* stackage, const std::string& lang,
                      const std::string& attrib,
                      std::vector<std::string>& flags)
 {
-  XMLElement* root = get_manifest_root(stackage);
-  for(XMLElement* ele = root->FirstChildElement(MANIFEST_TAG_EXPORT);
+  tinyxml2::XMLElement* root = get_manifest_root(stackage);
+  for(tinyxml2::XMLElement* ele = root->FirstChildElement(MANIFEST_TAG_EXPORT);
       ele;
       ele = ele->NextSiblingElement(MANIFEST_TAG_EXPORT))
   {
     bool os_match = false;
     const char *best_match = NULL;
-    for(XMLElement* ele2 = ele->FirstChildElement(lang.c_str());
+    for(tinyxml2::XMLElement* ele2 = ele->FirstChildElement(lang.c_str());
         ele2;
         ele2 = ele2->NextSiblingElement(lang.c_str()))
     {
@@ -1127,12 +1126,12 @@ Rosstackage::plugins(const std::string& name, const std::string& attrib,
       it != stackages.end();
       ++it)
   {
-    XMLElement* root = get_manifest_root(*it);
-    for(XMLElement* ele = root->FirstChildElement(MANIFEST_TAG_EXPORT);
+    tinyxml2::XMLElement* root = get_manifest_root(*it);
+    for(tinyxml2::XMLElement* ele = root->FirstChildElement(MANIFEST_TAG_EXPORT);
         ele;
         ele = ele->NextSiblingElement(MANIFEST_TAG_EXPORT))
     {
-      for(XMLElement* ele2 = ele->FirstChildElement(name.c_str());
+      for(tinyxml2::XMLElement* ele2 = ele->FirstChildElement(name.c_str());
           ele2;
           ele2 = ele2->NextSiblingElement(name.c_str()))
       {
@@ -1413,8 +1412,11 @@ Rosstackage::addStackage(const std::string& path)
   }
 
   // skip the stackage if it is not of correct type
-  if((manifest_name_ == ROSSTACK_MANIFEST_NAME && stackage->isPackage()) ||
-     (manifest_name_ == ROSPACK_MANIFEST_NAME && stackage->isStack()))
+  if( (stackage->is_wet_package_ &&
+       (manifest_name_ == ROSPACKAGE_MANIFEST_NAME)) ||
+      (!stackage->is_wet_package_ &&
+       (manifest_name_ == ROSSTACK_MANIFEST_NAME && stackage->isPackage()) ||
+       (manifest_name_ == ROSPACK_MANIFEST_NAME && stackage->isStack())) )
   {
     delete stackage;
     return;
@@ -1563,7 +1565,7 @@ Rosstackage::loadManifest(Stackage* stackage)
   if(stackage->manifest_loaded_)
     return;
 
-  if(stackage->manifest_.LoadFile(stackage->manifest_path_.c_str()) != XML_SUCCESS)
+  if(stackage->manifest_.LoadFile(stackage->manifest_path_.c_str()) != tinyxml2::XML_SUCCESS)
   {
     std::string errmsg = std::string("error parsing manifest of package ") +
             stackage->name_ + " at " + stackage->manifest_path_;
@@ -1609,11 +1611,11 @@ Rosstackage::computeDeps(Stackage* stackage, bool ignore_errors, bool ignore_mis
 void
 Rosstackage::computeDepsInternal(Stackage* stackage, bool ignore_errors, const std::string& depend_tag, bool ignore_missing)
 {
-  XMLElement* root;
+  tinyxml2::XMLElement* root;
   root = get_manifest_root(stackage);
 
   const char* dep_pkgname;
-  for(XMLElement *dep_ele = root->FirstChildElement(depend_tag.c_str());
+  for(tinyxml2::XMLElement *dep_ele = root->FirstChildElement(depend_tag.c_str());
       dep_ele;
       dep_ele = dep_ele->NextSiblingElement(depend_tag.c_str()))
   {
@@ -2338,10 +2340,10 @@ std::string Rosstack::get_manifest_type()
   return "stack";
 }
 
-XMLElement*
+tinyxml2::XMLElement*
 get_manifest_root(Stackage* stackage)
 {
-  XMLElement* ele = stackage->manifest_.RootElement();
+  tinyxml2::XMLElement* ele = stackage->manifest_.RootElement();
   if(!ele)
   {
     std::string errmsg = std::string("error parsing manifest of package ") +
